@@ -1,40 +1,48 @@
 # system configuration generated and used by the sysconfig module
-build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
+build_time_vars = {'ABIFLAGS': 'm',
+ 'AC_APPLE_UNIVERSAL_BUILD': 0,
  'AIX_GENUINE_CPLUSPLUS': 0,
  'AR': 'ar',
  'ARFLAGS': 'rc',
- 'ATHEOS_THREADS': 0,
+ 'ASDLGEN': '@echo python: not-found! cannot run ./Parser/asdl_c.py # ./Parser/asdl_c.py',
+ 'ASDLGEN_FILES': './Parser/asdl.py ./Parser/asdl_c.py',
+ 'AST_ASDL': './Parser/Python.asdl',
+ 'AST_C': 'Python/Python-ast.c',
+ 'AST_C_DIR': 'Python',
+ 'AST_H': 'Include/Python-ast.h',
+ 'AST_H_DIR': 'Include',
  'BASECFLAGS': '-fno-strict-aliasing',
+ 'BASECPPFLAGS': '',
  'BASEMODLIBS': '',
- 'BEOS_THREADS': 0,
  'BINDIR': '/usr/local/bin',
- 'BINLIBDEST': '/usr/local/lib/python2.7',
- 'BLDLIBRARY': '-L. -lpython2.7',
- 'BLDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/',
+ 'BINLIBDEST': '/usr/local/lib/python3.3',
+ 'BLDLIBRARY': '-L. -lpython3.3m',
+ 'BLDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/ -L/usr/obj/ports/Python-3.3.2/Python-3.3.2',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
+ 'BUILD_GNU_TYPE': 'sh-unknown-openbsd5.5',
+ 'BYTESTR_DEPS': '\\',
  'CC': 'cc -pthread',
  'CCSHARED': '-fPIC',
- 'CFLAGS': '-fno-strict-aliasing -O2 -pipe -DNDEBUG -O2 -pipe   -fPIC',
+ 'CFLAGS': '-fno-strict-aliasing -DNDEBUG -O2 -pipe  -fPIC -O2 -pipe -O2 -pipe',
  'CFLAGSFORSHARED': '-fPIC',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in',
- 'CONFIG_ARGS': "'--with-ensurepip=no' '--enable-shared' '--srcdir=/pobj/Python-2.7.18/Python-2.7.18' '--with-fpectl' '--with-threads' '--enable-ipv6' '--with-system-expat' '--with-system-ffi' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib/' 'CPPFLAGS=-I/usr/local/include'",
+ 'CONFIGURE_CFLAGS': '-O2 -pipe',
+ 'CONFIGURE_CPPFLAGS': '',
+ 'CONFIGURE_LDFLAGS': '-L/usr/local/lib/',
+ 'CONFIG_ARGS': "'--enable-shared' '--srcdir=/usr/obj/ports/Python-3.3.2/Python-3.3.2' '--with-fpectl' '--with-threads' '--enable-ipv6' '--with-cxx_main' '--with-system-expat' '--with-system-ffi' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib/'",
  'CONFINCLUDEDIR': '/usr/local/include',
- 'CONFINCLUDEPY': '/usr/local/include/python2.7',
- 'COREPYTHONPATH': ':plat-openbsd6:lib-tk:lib-old',
- 'COVERAGE_INFO': '/pobj/Python-2.7.18/Python-2.7.18/coverage.info',
- 'COVERAGE_REPORT': '/pobj/Python-2.7.18/Python-2.7.18/lcov-report',
- 'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov report"',
- 'CPPFLAGS': '-I. -IInclude -I./Include -I/usr/local/include',
+ 'CONFINCLUDEPY': '/usr/local/include/python3.3m',
+ 'COREPYTHONPATH': ':plat-openbsd5',
+ 'CPPFLAGS': '-I. -IInclude -I./Include',
  'CXX': 'c++ -pthread',
- 'C_THREADS': 0,
- 'DESTDIRS': '/usr/local /usr/local/lib /usr/local/lib/python2.7 /usr/local/lib/python2.7/lib-dynload',
- 'DESTLIB': '/usr/local/lib/python2.7',
+ 'DESTDIRS': '/usr/local /usr/local/lib /usr/local/lib/python3.3 /usr/local/lib/python3.3/lib-dynload',
+ 'DESTLIB': '/usr/local/lib/python3.3',
  'DESTPATH': '',
- 'DESTSHARED': '/usr/local/lib/python2.7/lib-dynload',
+ 'DESTSHARED': '/usr/local/lib/python3.3/lib-dynload',
  'DIRMODE': 755,
- 'DIST': 'README ChangeLog configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in Include Lib Misc Demo Ext-dummy',
- 'DISTDIRS': 'Include Lib Misc Demo Ext-dummy',
+ 'DIST': 'README ChangeLog configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in Include Lib Misc Ext-dummy',
+ 'DISTDIRS': 'Include Lib Misc Ext-dummy',
  'DISTFILES': 'README ChangeLog configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in',
  'DLINCLDIR': '.',
  'DLLLIBRARY': '',
@@ -43,26 +51,25 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 1,
  'DYNLOADFILE': 'dynload_shlib.o',
  'ENABLE_IPV6': 1,
- 'ENSUREPIP': 'no',
  'EXE': '',
  'EXEMODE': 755,
  'EXTRAMACHDEPPATH': '',
- 'EXTRAPLATDIR': '',
+ 'EXTRAPLATDIR': '@EXTRAPLATDIR@',
  'EXTRATESTOPTS': '',
  'EXTRA_CFLAGS': '',
+ 'EXT_SUFFIX': '.so',
  'FILEMODE': 644,
  'FLOCK_NEEDS_LIBBSD': 0,
  'GETPGRP_HAVE_ARG': 0,
  'GETTIMEOFDAY_NO_TZ': 0,
- 'GITBRANCH': '',
- 'GITTAG': '',
- 'GITVERSION': '',
- 'GLHACK': '-Dclear=__GLclear',
  'GNULD': 'yes',
+ 'GRAMMAR_C': 'Python/graminit.c',
+ 'GRAMMAR_H': 'Include/graminit.h',
+ 'GRAMMAR_INPUT': './Grammar/Grammar',
+ 'HAVE_ACCEPT4': 0,
  'HAVE_ACOSH': 1,
  'HAVE_ADDRINFO': 1,
  'HAVE_ALARM': 1,
- 'HAVE_ALLOCA_H': 0,
  'HAVE_ALTZONE': 0,
  'HAVE_ASINH': 1,
  'HAVE_ASM_TYPES_H': 0,
@@ -71,6 +78,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_BIND_TEXTDOMAIN_CODESET': 1,
  'HAVE_BLUETOOTH_BLUETOOTH_H': 0,
  'HAVE_BLUETOOTH_H': 0,
+ 'HAVE_BROKEN_MBSTOWCS': 0,
  'HAVE_BROKEN_NICE': 0,
  'HAVE_BROKEN_PIPE_BUF': 0,
  'HAVE_BROKEN_POLL': 0,
@@ -83,25 +91,18 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_CHOWN': 1,
  'HAVE_CHROOT': 1,
  'HAVE_CLOCK': 1,
+ 'HAVE_CLOCK_GETRES': 1,
+ 'HAVE_CLOCK_GETTIME': 1,
  'HAVE_COMPUTED_GOTOS': 1,
  'HAVE_CONFSTR': 1,
  'HAVE_CONIO_H': 0,
  'HAVE_COPYSIGN': 1,
- 'HAVE_CRYPT_H': 0,
  'HAVE_CTERMID': 1,
  'HAVE_CTERMID_R': 0,
- 'HAVE_CURSES_FILTER': 1,
  'HAVE_CURSES_H': 1,
- 'HAVE_CURSES_HAS_KEY': 1,
- 'HAVE_CURSES_IMMEDOK': 1,
- 'HAVE_CURSES_IS_PAD': 0,
  'HAVE_CURSES_IS_TERM_RESIZED': 1,
  'HAVE_CURSES_RESIZETERM': 1,
  'HAVE_CURSES_RESIZE_TERM': 1,
- 'HAVE_CURSES_SYNCOK': 1,
- 'HAVE_CURSES_TYPEAHEAD': 1,
- 'HAVE_CURSES_USE_ENV': 1,
- 'HAVE_CURSES_WCHGAT': 1,
  'HAVE_DECL_ISFINITE': 1,
  'HAVE_DECL_ISINF': 1,
  'HAVE_DECL_ISNAN': 1,
@@ -111,21 +112,28 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_DEV_PTMX': 0,
  'HAVE_DIRECT_H': 0,
  'HAVE_DIRENT_H': 1,
+ 'HAVE_DIRFD': 1,
  'HAVE_DLFCN_H': 1,
  'HAVE_DLOPEN': 1,
  'HAVE_DUP2': 1,
  'HAVE_DYNAMIC_LOADING': 1,
  'HAVE_EPOLL': 0,
+ 'HAVE_EPOLL_CREATE1': 0,
  'HAVE_ERF': 1,
  'HAVE_ERFC': 1,
  'HAVE_ERRNO_H': 1,
  'HAVE_EXECV': 1,
  'HAVE_EXPM1': 1,
+ 'HAVE_FACCESSAT': 1,
  'HAVE_FCHDIR': 1,
  'HAVE_FCHMOD': 1,
+ 'HAVE_FCHMODAT': 1,
  'HAVE_FCHOWN': 1,
+ 'HAVE_FCHOWNAT': 1,
  'HAVE_FCNTL_H': 1,
  'HAVE_FDATASYNC': 1,
+ 'HAVE_FDOPENDIR': 1,
+ 'HAVE_FEXECVE': 0,
  'HAVE_FINITE': 1,
  'HAVE_FLOCK': 1,
  'HAVE_FORK': 1,
@@ -133,19 +141,25 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_FPATHCONF': 1,
  'HAVE_FSEEK64': 0,
  'HAVE_FSEEKO': 1,
+ 'HAVE_FSTATAT': 1,
  'HAVE_FSTATVFS': 1,
  'HAVE_FSYNC': 1,
  'HAVE_FTELL64': 0,
  'HAVE_FTELLO': 1,
  'HAVE_FTIME': 0,
  'HAVE_FTRUNCATE': 1,
+ 'HAVE_FUTIMENS': 1,
+ 'HAVE_FUTIMES': 1,
+ 'HAVE_FUTIMESAT': 0,
  'HAVE_GAI_STRERROR': 1,
  'HAVE_GAMMA': 1,
- 'HAVE_GCC_ASM_FOR_X87': 1,
+ 'HAVE_GCC_ASM_FOR_X64': 0,
+ 'HAVE_GCC_ASM_FOR_X87': 0,
+ 'HAVE_GCC_UINT128_T': 0,
  'HAVE_GETADDRINFO': 1,
  'HAVE_GETCWD': 1,
  'HAVE_GETC_UNLOCKED': 1,
- 'HAVE_GETENTROPY': 1,
+ 'HAVE_GETGROUPLIST': 1,
  'HAVE_GETGROUPS': 1,
  'HAVE_GETHOSTBYNAME': 1,
  'HAVE_GETHOSTBYNAME_R': 0,
@@ -170,10 +184,12 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_GETSPNAM': 0,
  'HAVE_GETTIMEOFDAY': 1,
  'HAVE_GETWD': 1,
+ 'HAVE_GLIBC_MEMMOVE_BUG': 0,
  'HAVE_GRP_H': 1,
  'HAVE_HSTRERROR': 1,
  'HAVE_HYPOT': 1,
  'HAVE_IEEEFP_H': 1,
+ 'HAVE_IF_NAMEINDEX': 1,
  'HAVE_INET_ATON': 1,
  'HAVE_INET_PTON': 1,
  'HAVE_INITGROUPS': 1,
@@ -181,6 +197,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_INT64_T': 1,
  'HAVE_INTTYPES_H': 1,
  'HAVE_IO_H': 0,
+ 'HAVE_IPA_PURE_CONST_BUG': 0,
  'HAVE_KILL': 1,
  'HAVE_KILLPG': 1,
  'HAVE_KQUEUE': 1,
@@ -193,48 +210,69 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_LIBDL': 0,
  'HAVE_LIBDLD': 0,
  'HAVE_LIBIEEE': 0,
- 'HAVE_LIBINTL_H': 1,
+ 'HAVE_LIBINTL_H': 0,
  'HAVE_LIBREADLINE': 1,
  'HAVE_LIBRESOLV': 0,
+ 'HAVE_LIBSENDFILE': 0,
  'HAVE_LIBUTIL_H': 0,
  'HAVE_LINK': 1,
+ 'HAVE_LINKAT': 1,
+ 'HAVE_LINUX_CAN_H': 0,
+ 'HAVE_LINUX_CAN_RAW_H': 0,
  'HAVE_LINUX_NETLINK_H': 0,
  'HAVE_LINUX_TIPC_H': 0,
+ 'HAVE_LOCKF': 1,
  'HAVE_LOG1P': 1,
+ 'HAVE_LOG2': 1,
  'HAVE_LONG_DOUBLE': 1,
  'HAVE_LONG_LONG': 1,
  'HAVE_LSTAT': 1,
+ 'HAVE_LUTIMES': 0,
  'HAVE_MAKEDEV': 1,
+ 'HAVE_MBRTOWC': 1,
  'HAVE_MEMMOVE': 1,
  'HAVE_MEMORY_H': 1,
+ 'HAVE_MEMRCHR': 1,
+ 'HAVE_MKDIRAT': 1,
  'HAVE_MKFIFO': 1,
+ 'HAVE_MKFIFOAT': 1,
  'HAVE_MKNOD': 1,
+ 'HAVE_MKNODAT': 1,
  'HAVE_MKTIME': 1,
  'HAVE_MMAP': 1,
  'HAVE_MREMAP': 0,
  'HAVE_NCURSES_H': 1,
  'HAVE_NDIR_H': 0,
  'HAVE_NETPACKET_PACKET_H': 0,
+ 'HAVE_NET_IF_H': 1,
  'HAVE_NICE': 1,
+ 'HAVE_OPENAT': 1,
  'HAVE_OPENPTY': 1,
  'HAVE_OSX105_SDK': 0,
  'HAVE_PATHCONF': 1,
  'HAVE_PAUSE': 1,
+ 'HAVE_PIPE2': 0,
  'HAVE_PLOCK': 0,
  'HAVE_POLL': 1,
  'HAVE_POLL_H': 1,
+ 'HAVE_POSIX_FADVISE': 0,
+ 'HAVE_POSIX_FALLOCATE': 0,
+ 'HAVE_PREAD': 1,
  'HAVE_PROCESS_H': 0,
  'HAVE_PROTOTYPES': 1,
- 'HAVE_PTH': 0,
- 'HAVE_PTHREAD_ATFORK': 1,
  'HAVE_PTHREAD_DESTRUCTOR': 0,
  'HAVE_PTHREAD_H': 1,
  'HAVE_PTHREAD_INIT': 0,
+ 'HAVE_PTHREAD_KILL': 1,
  'HAVE_PTHREAD_SIGMASK': 1,
  'HAVE_PTY_H': 0,
  'HAVE_PUTENV': 1,
+ 'HAVE_PWRITE': 1,
  'HAVE_READLINK': 1,
+ 'HAVE_READLINKAT': 1,
+ 'HAVE_READV': 1,
  'HAVE_REALPATH': 1,
+ 'HAVE_RENAMEAT': 1,
  'HAVE_RL_CALLBACK': 1,
  'HAVE_RL_CATCH_SIGNAL': 1,
  'HAVE_RL_COMPLETION_APPEND_CHARACTER': 1,
@@ -242,21 +280,29 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_RL_COMPLETION_MATCHES': 1,
  'HAVE_RL_COMPLETION_SUPPRESS_APPEND': 1,
  'HAVE_RL_PRE_INPUT_HOOK': 1,
- 'HAVE_RL_RESIZE_TERMINAL': 1,
  'HAVE_ROUND': 1,
+ 'HAVE_SCHED_GET_PRIORITY_MAX': 1,
+ 'HAVE_SCHED_H': 1,
+ 'HAVE_SCHED_RR_GET_INTERVAL': 0,
+ 'HAVE_SCHED_SETAFFINITY': 0,
+ 'HAVE_SCHED_SETPARAM': 0,
+ 'HAVE_SCHED_SETSCHEDULER': 0,
  'HAVE_SELECT': 1,
  'HAVE_SEM_GETVALUE': 1,
  'HAVE_SEM_OPEN': 1,
  'HAVE_SEM_TIMEDWAIT': 1,
  'HAVE_SEM_UNLINK': 1,
+ 'HAVE_SENDFILE': 0,
  'HAVE_SETEGID': 1,
  'HAVE_SETEUID': 1,
  'HAVE_SETGID': 1,
  'HAVE_SETGROUPS': 1,
+ 'HAVE_SETHOSTNAME': 1,
  'HAVE_SETITIMER': 1,
  'HAVE_SETLOCALE': 1,
  'HAVE_SETPGID': 1,
  'HAVE_SETPGRP': 1,
+ 'HAVE_SETPRIORITY': 1,
  'HAVE_SETREGID': 1,
  'HAVE_SETRESGID': 1,
  'HAVE_SETRESUID': 1,
@@ -266,9 +312,14 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_SETVBUF': 1,
  'HAVE_SHADOW_H': 0,
  'HAVE_SIGACTION': 1,
+ 'HAVE_SIGALTSTACK': 1,
  'HAVE_SIGINTERRUPT': 1,
  'HAVE_SIGNAL_H': 1,
+ 'HAVE_SIGPENDING': 1,
  'HAVE_SIGRELSE': 0,
+ 'HAVE_SIGTIMEDWAIT': 0,
+ 'HAVE_SIGWAIT': 1,
+ 'HAVE_SIGWAITINFO': 0,
  'HAVE_SNPRINTF': 1,
  'HAVE_SOCKADDR_SA_LEN': 1,
  'HAVE_SOCKADDR_STORAGE': 1,
@@ -285,6 +336,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_STRFTIME': 1,
  'HAVE_STRINGS_H': 1,
  'HAVE_STRING_H': 1,
+ 'HAVE_STRLCPY': 1,
  'HAVE_STROPTS_H': 0,
  'HAVE_STRUCT_STAT_ST_BIRTHTIME': 0,
  'HAVE_STRUCT_STAT_ST_BLKSIZE': 1,
@@ -293,15 +345,21 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_STRUCT_STAT_ST_GEN': 1,
  'HAVE_STRUCT_STAT_ST_RDEV': 1,
  'HAVE_STRUCT_TM_TM_ZONE': 1,
+ 'HAVE_ST_BLOCKS': 1,
  'HAVE_SYMLINK': 1,
+ 'HAVE_SYMLINKAT': 1,
+ 'HAVE_SYNC': 1,
  'HAVE_SYSCONF': 1,
  'HAVE_SYSEXITS_H': 1,
  'HAVE_SYS_AUDIOIO_H': 1,
  'HAVE_SYS_BSDTTY_H': 0,
+ 'HAVE_SYS_DEVPOLL_H': 0,
  'HAVE_SYS_DIR_H': 0,
  'HAVE_SYS_EPOLL_H': 0,
  'HAVE_SYS_EVENT_H': 1,
  'HAVE_SYS_FILE_H': 1,
+ 'HAVE_SYS_IOCTL_H': 1,
+ 'HAVE_SYS_KERN_CONTROL_H': 0,
  'HAVE_SYS_LOADAVG_H': 0,
  'HAVE_SYS_LOCK_H': 1,
  'HAVE_SYS_MKDEV_H': 0,
@@ -309,27 +367,29 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_SYS_NDIR_H': 0,
  'HAVE_SYS_PARAM_H': 1,
  'HAVE_SYS_POLL_H': 1,
- 'HAVE_SYS_RANDOM_H': 0,
  'HAVE_SYS_RESOURCE_H': 1,
  'HAVE_SYS_SELECT_H': 1,
+ 'HAVE_SYS_SENDFILE_H': 0,
  'HAVE_SYS_SOCKET_H': 1,
  'HAVE_SYS_STATVFS_H': 1,
  'HAVE_SYS_STAT_H': 1,
- 'HAVE_SYS_SYSMACROS_H': 0,
+ 'HAVE_SYS_SYSCALL_H': 1,
+ 'HAVE_SYS_SYS_DOMAIN_H': 0,
  'HAVE_SYS_TERMIO_H': 0,
  'HAVE_SYS_TIMES_H': 1,
  'HAVE_SYS_TIME_H': 1,
  'HAVE_SYS_TYPES_H': 1,
+ 'HAVE_SYS_UIO_H': 1,
  'HAVE_SYS_UN_H': 1,
  'HAVE_SYS_UTSNAME_H': 1,
  'HAVE_SYS_WAIT_H': 1,
+ 'HAVE_SYS_XATTR_H': 0,
  'HAVE_TCGETPGRP': 1,
  'HAVE_TCSETPGRP': 1,
  'HAVE_TEMPNAM': 1,
  'HAVE_TERMIOS_H': 1,
  'HAVE_TERM_H': 1,
  'HAVE_TGAMMA': 1,
- 'HAVE_THREAD_H': 0,
  'HAVE_TIMEGM': 1,
  'HAVE_TIMES': 1,
  'HAVE_TMPFILE': 1,
@@ -344,79 +404,89 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_UINTPTR_T': 1,
  'HAVE_UNAME': 1,
  'HAVE_UNISTD_H': 1,
+ 'HAVE_UNLINKAT': 1,
  'HAVE_UNSETENV': 1,
  'HAVE_USABLE_WCHAR_T': 0,
  'HAVE_UTIL_H': 1,
+ 'HAVE_UTIMENSAT': 1,
  'HAVE_UTIMES': 1,
  'HAVE_UTIME_H': 1,
  'HAVE_WAIT3': 1,
  'HAVE_WAIT4': 1,
+ 'HAVE_WAITID': 0,
  'HAVE_WAITPID': 1,
  'HAVE_WCHAR_H': 1,
  'HAVE_WCSCOLL': 1,
+ 'HAVE_WCSFTIME': 1,
+ 'HAVE_WCSXFRM': 1,
  'HAVE_WORKING_TZSET': 1,
+ 'HAVE_WRITEV': 1,
  'HAVE_ZLIB_COPY': 1,
  'HAVE__GETPTY': 0,
- 'HOST_GNU_TYPE': 'i386-unknown-openbsd6.7',
- 'HURD_C_THREADS': 0,
- 'INCLDIRSTOMAKE': '/usr/local/include /usr/local/include /usr/local/include/python2.7 /usr/local/include/python2.7',
+ 'HGBRANCH': '',
+ 'HGTAG': '',
+ 'HGVERSION': '',
+ 'HOST_GNU_TYPE': 'sh-unknown-openbsd5.5',
+ 'INCLDIRSTOMAKE': '/usr/local/include /usr/local/include /usr/local/include/python3.3m /usr/local/include/python3.3m',
  'INCLUDEDIR': '/usr/local/include',
- 'INCLUDEPY': '/usr/local/include/python2.7',
- 'INSTALL': '/pobj/Python-2.7.18/bin/install -c',
- 'INSTALL_DATA': '/pobj/Python-2.7.18/bin/install -c -m 644',
- 'INSTALL_PROGRAM': '/pobj/Python-2.7.18/bin/install -c -s -m 755',
- 'INSTALL_SCRIPT': '/pobj/Python-2.7.18/bin/install -c -m 755',
- 'INSTALL_SHARED': '/pobj/Python-2.7.18/bin/install -c -m 555',
- 'INSTSONAME': 'libpython2.7.so.0.0',
+ 'INCLUDEPY': '/usr/local/include/python3.3m',
+ 'INSTALL': '/usr/bin/install -c -o root -g bin',
+ 'INSTALL_DATA': 'install -c -o root -g bin -m 444',
+ 'INSTALL_PROGRAM': 'install -c -s -o root -g bin -m 555',
+ 'INSTALL_SCRIPT': 'install -c -o root -g bin -m 555',
+ 'INSTALL_SHARED': '/usr/bin/install -c -o root -g bin -m 555',
+ 'INSTSONAME': 'libpython3.3m.so.0.0',
+ 'IO_H': 'Modules/_io/_iomodule.h',
+ 'IO_OBJS': '\\',
  'LDCXXSHARED': 'c++ -pthread -shared -fPIC',
- 'LDFLAGS': '-L/usr/local/lib/',
+ 'LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.3.2/Python-3.3.2',
  'LDLAST': '',
- 'LDLIBRARY': 'libpython2.7.so',
+ 'LDLIBRARY': 'libpython3.3m.so',
  'LDLIBRARYDIR': '',
- 'LDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/',
+ 'LDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/ -L/usr/obj/ports/Python-3.3.2/Python-3.3.2',
+ 'LDVERSION': '3.3m',
  'LIBC': '',
- 'LIBDEST': '/usr/local/lib/python2.7',
+ 'LIBDEST': '/usr/local/lib/python3.3',
  'LIBDIR': '/usr/local/lib',
  'LIBFFI_INCLUDEDIR': '/usr/local/include',
  'LIBM': '-lm',
  'LIBOBJDIR': 'Python/',
  'LIBOBJS': '',
- 'LIBP': '/usr/local/lib/python2.7',
  'LIBPC': '/usr/local/lib/pkgconfig',
- 'LIBPL': '/usr/local/lib/python2.7/config',
- 'LIBRARY': 'libpython2.7.a',
+ 'LIBPL': '/usr/local/lib/python3.3/config-3.3m',
+ 'LIBRARY': 'libpython3.3m.a',
  'LIBRARY_OBJS': '\\',
+ 'LIBRARY_OBJS_OMIT_FROZEN': '\\',
  'LIBS': '-lintl -lpthread  -lutil',
- 'LIBSUBDIRS': 'lib-tk lib-tk/test lib-tk/test/test_tkinter \\',
- 'LINKCC': 'cc -pthread',
+ 'LIBSUBDIRS': 'tkinter tkinter/test tkinter/test/test_tkinter \\',
+ 'LINKCC': 'c++ -pthread',
  'LINKFORSHARED': '-Wl,--export-dynamic',
- 'LLVM_PROF_ERR': 'no',
- 'LLVM_PROF_FILE': '',
- 'LLVM_PROF_MERGER': '',
  'LN': 'ln',
  'LOCALMODLIBS': '',
- 'MACHDEP': 'openbsd6',
- 'MACHDEPPATH': ':plat-openbsd6',
- 'MACHDEPS': 'plat-openbsd6',
+ 'LOG1P_DROPS_ZERO_SIGN': 0,
+ 'MACHDEP': 'openbsd5',
+ 'MACHDEPPATH': ':plat-openbsd5',
+ 'MACHDEPS': 'plat-openbsd5 @EXTRAPLATDIR@',
  'MACHDEP_OBJS': '',
- 'MACHDESTLIB': '/usr/local/lib/python2.7',
- 'MACH_C_THREADS': 0,
+ 'MACHDESTLIB': '/usr/local/lib/python3.3',
  'MACOSX_DEPLOYMENT_TARGET': '',
- 'MAINCC': 'cc -pthread',
+ 'MAINCC': 'c++ -pthread',
  'MAJOR_IN_MKDEV': 0,
  'MAJOR_IN_SYSMACROS': 0,
  'MAKESETUP': './Modules/makesetup',
  'MANDIR': '/usr/local/man',
- 'MEMTESTOPTS': '-l -x test_subprocess test_io test_lib2to3 \\ -x test_dl test___all__ test_fork1 \\',
  'MKDIR_P': 'mkdir -p',
  'MODLIBS': '',
- 'MODOBJS': 'Modules/threadmodule.o  Modules/signalmodule.o  Modules/posixmodule.o  Modules/errnomodule.o  Modules/pwdmodule.o  Modules/_sre.o  Modules/_codecsmodule.o  Modules/_weakref.o  Modules/zipimport.o  Modules/symtablemodule.o  Modules/xxsubtype.o',
+ 'MODOBJS': 'Modules/_threadmodule.o  Modules/signalmodule.o  Modules/posixmodule.o  Modules/errnomodule.o  Modules/pwdmodule.o  Modules/_sre.o  Modules/_codecsmodule.o  Modules/_weakref.o  Modules/_functoolsmodule.o  Modules/operator.o  Modules/_collectionsmodule.o  Modules/itertoolsmodule.o  Modules/_localemodule.o  Modules/_iomodule.o Modules/iobase.o Modules/fileio.o Modules/bytesio.o Modules/bufferedio.o Modules/textio.o Modules/stringio.o  Modules/zipimport.o  Modules/faulthandler.o  Modules/symtablemodule.o  Modules/xxsubtype.o',
  'MODULE_OBJS': '\\',
  'MULTIARCH': '',
  'MVWDELCH_IS_EXPRESSION': 1,
+ 'NO_AS_NEEDED': '-Wl,--no-as-needed',
  'OBJECT_OBJS': '\\',
- 'OLDPATH': ':lib-old',
- 'OPT': '-DNDEBUG -O2 -pipe   -fPIC',
+ 'OPCODETARGETGEN': '\\',
+ 'OPCODETARGETGEN_FILES': '\\',
+ 'OPCODETARGETS_H': '\\',
+ 'OPT': '-DNDEBUG -O2 -pipe  -fPIC',
  'OTHER_LIBTOOL_OPT': '',
  'PACKAGE_BUGREPORT': 0,
  'PACKAGE_NAME': 0,
@@ -425,53 +495,50 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'PACKAGE_URL': 0,
  'PACKAGE_VERSION': 0,
  'PARSER_HEADERS': '\\',
- 'PARSER_OBJS': '\\ Parser/myreadline.o Parser/tokenizer.o',
+ 'PARSER_OBJS': '\\ Parser/myreadline.o Parser/parsetok.o Parser/tokenizer.o',
  'PGEN': 'Parser/pgen',
  'PGENOBJS': '\\ \\',
  'PGENSRCS': '\\ \\',
  'PGOBJS': '\\',
- 'PGO_PROF_GEN_FLAG': '',
- 'PGO_PROF_USE_FLAG': '',
  'PGSRCS': '\\',
- 'PLATDIR': 'plat-openbsd6',
- 'PLATMACDIRS': 'plat-mac plat-mac/Carbon plat-mac/lib-scriptpackages \\',
- 'PLATMACPATH': ':plat-mac:plat-mac/lib-scriptpackages',
+ 'PLATDIR': 'plat-openbsd5',
  'POBJS': '\\',
  'POSIX_SEMAPHORES_NOT_ENABLED': 0,
- 'PROFILE_TASK': '-m test.regrtest --pgo -x test_asyncore test_gdb test_multiprocessing test_subprocess',
+ 'PROFILE_TASK': './Tools/pybench/pybench.py -n 2 --with-gc --with-syscheck',
  'PSRCS': '\\',
  'PTHREAD_SYSTEM_SCHED_SUPPORTED': 1,
  'PURIFY': '',
+ 'PY3LIBRARY': 'libpython3.so',
  'PYLONG_BITS_IN_DIGIT': 0,
  'PYTHON': 'python',
  'PYTHONFRAMEWORK': '',
  'PYTHONFRAMEWORKDIR': 'no-framework',
  'PYTHONFRAMEWORKINSTALLDIR': '',
  'PYTHONFRAMEWORKPREFIX': '',
- 'PYTHONPATH': ':plat-openbsd6:lib-tk:lib-old',
+ 'PYTHONPATH': ':plat-openbsd5',
  'PYTHON_FOR_BUILD': './python -E',
- 'PYTHON_FOR_REGEN': 'python3',
  'PYTHON_HEADERS': '\\',
  'PYTHON_OBJS': '\\',
- 'PY_CFLAGS': '-fno-strict-aliasing -O2 -pipe -DNDEBUG -O2 -pipe   -fPIC -I. -IInclude -I./Include -I/usr/local/include -fPIC -DPy_BUILD_CORE',
+ 'PY_CFLAGS': '-fno-strict-aliasing -DNDEBUG -O2 -pipe  -fPIC -O2 -pipe -O2 -pipe',
+ 'PY_CORE_CFLAGS': '-fno-strict-aliasing -DNDEBUG -O2 -pipe  -fPIC -O2 -pipe -O2 -pipe -I. -IInclude -I./Include -fPIC -DPy_BUILD_CORE',
+ 'PY_CPPFLAGS': '-I. -IInclude -I./Include',
  'PY_FORMAT_LONG_LONG': '"ll"',
  'PY_FORMAT_SIZE_T': '"z"',
- 'PY_UNICODE_TYPE': 'unsigned short',
+ 'PY_LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.3.2/Python-3.3.2',
  'Py_DEBUG': 0,
  'Py_ENABLE_SHARED': 1,
- 'Py_UNICODE_SIZE': 2,
- 'Py_USING_UNICODE': 1,
- 'QUICKTESTOPTS': '-l -x test_subprocess test_io test_lib2to3 \\',
+ 'QUICKTESTOPTS': '-x test_subprocess test_io test_lib2to3 \\',
  'RANLIB': 'ranlib',
+ 'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/pobj/Python-2.7.18/Python-2.7.18',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.3.2/Python-3.3.2:',
  'SCRIPTDIR': '/usr/local/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SGI_ABI': '',
  'SHELL': '/bin/sh',
  'SHLIBS': '-lintl -lpthread  -lutil',
- 'SHLIB_EXT': '".so"',
+ 'SHLIB_SUFFIX': '.so',
  'SIGNAL_OBJS': '',
  'SIGNED_RIGHT_SHIFT_ZERO_FILLS': 0,
  'SITEPATH': '',
@@ -480,7 +547,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'SIZEOF_FPOS_T': 8,
  'SIZEOF_INT': 4,
  'SIZEOF_LONG': 4,
- 'SIZEOF_LONG_DOUBLE': 12,
+ 'SIZEOF_LONG_DOUBLE': 8,
  'SIZEOF_LONG_LONG': 8,
  'SIZEOF_OFF_T': 8,
  'SIZEOF_PID_T': 4,
@@ -493,35 +560,33 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'SIZEOF_WCHAR_T': 4,
  'SIZEOF__BOOL': 1,
  'SO': '.so',
- 'SRCDIRS': 'Parser Objects Python Modules Modules/_io',
+ 'SOABI': 'cpython-33m',
+ 'SRCDIRS': 'Parser Grammar Objects Python Modules Mac',
  'SRC_GDB_HOOKS': './Tools/gdb/libpython.py',
  'STDC_HEADERS': 1,
  'STRICT_SYSV_CURSES': "/* Don't use ncurses extensions */",
- 'STRINGLIB_HEADERS': '\\',
  'SUBDIRS': '',
- 'SUBDIRSTOO': 'Include Lib Misc Demo',
+ 'SUBDIRSTOO': 'Include Lib Misc',
  'SYSLIBS': '-lm',
  'SYS_SELECT_WITH_SYS_TIME': 1,
  'TANH_PRESERVES_ZERO_SIGN': 1,
- 'TCLTK_INCLUDES': '',
- 'TCLTK_LIBS': '',
- 'TESTOPTS': '-l',
+ 'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPROG': './Lib/test/regrtest.py',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/pobj/Python-2.7.18/Python-2.7.18 ./python -Wd -3 -E -tt',
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.3.2/Python-3.3.2: ./python',
  'TESTPYTHONOPTS': '',
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.3.2/Python-3.3.2: ./python ./Tools/scripts/run_tests.py',
+ 'TESTTIMEOUT': 3600,
  'THREADOBJ': 'Python/thread.o',
+ 'TIMEMODULE_LIB': 0,
  'TIME_WITH_SYS_TIME': 1,
- 'TKPATH': ':lib-tk',
  'TM_IN_SYS_TIME': 0,
- 'UNICODE_OBJS': 'Objects/unicodeobject.o Objects/unicodectype.o',
+ 'UNICODE_DEPS': '\\ \\',
  'UNIVERSALSDK': '',
  'USE_COMPUTED_GOTOS': 0,
- 'USE_TOOLBOX_OBJECT_GLUE': 0,
+ 'USE_INLINE': 1,
  'VA_LIST_IS_ARRAY': 0,
- 'VERSION': '2.7',
+ 'VERSION': '3.3',
  'WANT_SIGFPE_HANDLER': 1,
- 'WANT_WCTYPE_FUNCTIONS': 0,
  'WINDOW_HAS_FLAGS': 1,
  'WITH_DOC_STRINGS': 1,
  'WITH_DYLD': 0,
@@ -531,13 +596,11 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'WITH_THREAD': 1,
  'WITH_TSC': 0,
  'WITH_VALGRIND': 0,
- 'X87_DOUBLE_ROUNDING': 1,
+ 'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/pobj/Python-2.7.18/Python-2.7.18',
- 'abs_srcdir': '/pobj/Python-2.7.18/Python-2.7.18',
- 'build': 'i386-unknown-openbsd6.7',
+ 'abs_builddir': '/usr/obj/ports/Python-3.3.2/Python-3.3.2',
+ 'abs_srcdir': '/usr/obj/ports/Python-3.3.2/Python-3.3.2',
  'datarootdir': '/usr/local/share',
  'exec_prefix': '/usr/local',
- 'host': 'i386-unknown-openbsd6.7',
  'prefix': '/usr/local',
  'srcdir': '.'}
